#include "luminosity.hh"
#include "QCD_EW.hh"
#include "math/integration.hh"
#include "LHAPDF/LHAPDF.h"


using namespace std;



namespace lumi {

  // Higgs part
  double lumi_gg(double x1, double x2, double mu) {
    return LHAPDF::xfx(x1,mu,0)/x1 * LHAPDF::xfx(x2,mu,0)/x2;
  }





  // Drell-Yan part
  //
  const double V_ud = EW::vud;//0.975;
  const double V_us = EW::vus;//0.222;
  const double V_cd = EW::vcd;//0.222;
  const double V_cs = EW::vcs;//0.974;
  const double V_ud_sq = V_ud*V_ud;     double V_us_sq = V_us*V_us;
  const double V_cd_sq = V_cd*V_cd;     double V_cs_sq = V_cs*V_cs;
  const double V_ub_sq = 1. - V_ud_sq - V_us_sq;
  const double V_cb_sq = 1. - V_cd_sq - V_cs_sq;
  const double V_td_sq = 1. - V_ud_sq - V_cd_sq;
  const double V_ts_sq = 1. - V_us_sq - V_cs_sq;
  const double V_tb_sq = 1. - V_ub_sq - V_cb_sq;
  //
  double N_Zgamma(double Q, double alpha){
    double Qsq = Q*Q;  double mZsq = MASS::mass_Z*MASS::mass_Z;
    return (1.-4.*EW::s2W)/8./EW::s2W/EW::c2W * Qsq*(Qsq-mZsq)/((Qsq-mZsq)*(Qsq-mZsq) + EW::Gamma_Z*EW::Gamma_Z * mZsq);
  }
  double N_Z(double Q, double alpha){
    double Qsq = Q*Q;  double mZsq = MASS::mass_Z*MASS::mass_Z;
    return 3./16./EW::s2W/EW::c2W/alpha * EW::Gamma_Z * EW::Br_l_Z/MASS::mass_Z
      * Qsq*Qsq/((Qsq-mZsq)*(Qsq-mZsq) + EW::Gamma_Z*EW::Gamma_Z * mZsq);
  }
  const double a_u_Z = -1., a_d_Z = 1.;
  const double v_u_Z = 1.-8./3.*EW::s2W, v_d_Z = -1.+4./3.*EW::s2W;
  double VsqAsq_u(double M, double alpha){
    return EW::e_u*EW::e_u + N_Zgamma(M,alpha)*EW::e_u*v_u_Z + N_Z(M,alpha)*(v_u_Z*v_u_Z + a_u_Z*a_u_Z);
  }
  double VsqAsq_d(double M, double alpha){
    return EW::e_d*EW::e_d + N_Zgamma(M,alpha)*EW::e_d*v_d_Z + N_Z(M,alpha)*(v_d_Z*v_d_Z + a_d_Z*a_d_Z) ;
  }
  double lumi_DYn(double x1, double x2, double mu, double M, collider c) { // neutral (Z+gamma)
    //double alpha = EW::alpha_QED(M);
    double alpha = EW::alpha_em_Z;
    double pref_u = VsqAsq_u(M,alpha);
    double pref_d = VsqAsq_d(M,alpha);
    double xfx1[13], xfx2[13];
    LHAPDF::xfx(x1, mu, xfx1);
    LHAPDF::xfx(x2, mu, xfx2);
    double res = 0;
    if (c==pp) {
      res = pref_u * ( xfx1[6+2]*xfx2[6-2] + xfx1[6-2]*xfx2[6+2] // up
		       + xfx1[6+4]*xfx2[6-4] + xfx1[6-4]*xfx2[6+4] // charm
		       + xfx1[6+6]*xfx2[6-6] + xfx1[6-6]*xfx2[6+6] // top
		       )
	+ pref_d * ( xfx1[6+1]*xfx2[6-1] + xfx1[6-1]*xfx2[6+1] // down
		     + xfx1[6+3]*xfx2[6-3] + xfx1[6-3]*xfx2[6+3] // strange
		     + xfx1[6+5]*xfx2[6-5] + xfx1[6-5]*xfx2[6+5] // bottom
		     );
    } else if (c==ppbar) {
      res = pref_u * ( xfx1[6+2]*xfx2[6+2] + xfx1[6-2]*xfx2[6-2] // up
		       + xfx1[6+4]*xfx2[6+4] + xfx1[6-4]*xfx2[6-4] // charm
		       + xfx1[6+6]*xfx2[6+6] + xfx1[6-6]*xfx2[6-6] // top
		       )
	+ pref_d * ( xfx1[6+1]*xfx2[6+1] + xfx1[6-1]*xfx2[6-1] // down
		     + xfx1[6+3]*xfx2[6+3] + xfx1[6-3]*xfx2[6-3] // strange
		     + xfx1[6+5]*xfx2[6+5] + xfx1[6-5]*xfx2[6-5] // bottom
		     );
    }
    return res/x1/x2;
  }
  double lumi_DYp(double x1, double x2, double mu, double M, collider c) { // positive (W+)
    double xfx1[13], xfx2[13];
    LHAPDF::xfx(x1, mu, xfx1);
    LHAPDF::xfx(x2, mu, xfx2);
    double res = 0;
    if (c==pp) {
      res = 0
	+ xfx1[6+2] * ( V_ud_sq * xfx2[6-1] + V_us_sq * xfx2[6-3] + V_ub_sq * xfx2[6-5] )
	+ xfx1[6+4] * ( V_cd_sq * xfx2[6-1] + V_cs_sq * xfx2[6-3] + V_cb_sq * xfx2[6-5] )
	+ xfx1[6+6] * ( V_td_sq * xfx2[6-1] + V_ts_sq * xfx2[6-3] + V_tb_sq * xfx2[6-5] )
	+ xfx2[6+2] * ( V_ud_sq * xfx1[6-1] + V_us_sq * xfx1[6-3] + V_ub_sq * xfx1[6-5] )
	+ xfx2[6+4] * ( V_cd_sq * xfx1[6-1] + V_cs_sq * xfx1[6-3] + V_cb_sq * xfx1[6-5] )
	+ xfx2[6+6] * ( V_td_sq * xfx1[6-1] + V_ts_sq * xfx1[6-3] + V_tb_sq * xfx1[6-5] );
    } else if (c==ppbar) {
      res = 0
	+ xfx1[6+2] * ( V_ud_sq * xfx2[6+1] + V_us_sq * xfx2[6+3] + V_ub_sq * xfx2[6+5] )
	+ xfx1[6+4] * ( V_cd_sq * xfx2[6+1] + V_cs_sq * xfx2[6+3] + V_cb_sq * xfx2[6+5] )
	+ xfx1[6+6] * ( V_td_sq * xfx2[6+1] + V_ts_sq * xfx2[6+3] + V_tb_sq * xfx2[6+5] )
	+ xfx2[6-2] * ( V_ud_sq * xfx1[6-1] + V_us_sq * xfx1[6-3] + V_ub_sq * xfx1[6-5] )
	+ xfx2[6-4] * ( V_cd_sq * xfx1[6-1] + V_cs_sq * xfx1[6-3] + V_cb_sq * xfx1[6-5] )
	+ xfx2[6-6] * ( V_td_sq * xfx1[6-1] + V_ts_sq * xfx1[6-3] + V_tb_sq * xfx1[6-5] );
    }
    return res/x1/x2;
  }
  double lumi_DYm(double x1, double x2, double mu, double M, collider c) { // negative (W-)
    double xfx1[13], xfx2[13];
    LHAPDF::xfx(x1, mu, xfx1);
    LHAPDF::xfx(x2, mu, xfx2);
    double res = 0;
    if (c==pp) {
      res = 0
	+ xfx1[6-2] * ( V_ud_sq * xfx2[6+1] + V_us_sq * xfx2[6+3] + V_ub_sq * xfx2[6+5] )
	+ xfx1[6-4] * ( V_cd_sq * xfx2[6+1] + V_cs_sq * xfx2[6+3] + V_cb_sq * xfx2[6+5] )
	+ xfx1[6-6] * ( V_td_sq * xfx2[6+1] + V_ts_sq * xfx2[6+3] + V_tb_sq * xfx2[6+5] )
	+ xfx2[6-2] * ( V_ud_sq * xfx1[6+1] + V_us_sq * xfx1[6+3] + V_ub_sq * xfx1[6+5] )
	+ xfx2[6-4] * ( V_cd_sq * xfx1[6+1] + V_cs_sq * xfx1[6+3] + V_cb_sq * xfx1[6+5] )
	+ xfx2[6-6] * ( V_td_sq * xfx1[6+1] + V_ts_sq * xfx1[6+3] + V_tb_sq * xfx1[6+5] );
    } else if (c==ppbar) {
      res = 0
	+ xfx1[6-2] * ( V_ud_sq * xfx2[6-1] + V_us_sq * xfx2[6-3] + V_ub_sq * xfx2[6-5] )
	+ xfx1[6-4] * ( V_cd_sq * xfx2[6-1] + V_cs_sq * xfx2[6-3] + V_cb_sq * xfx2[6-5] )
	+ xfx1[6-6] * ( V_td_sq * xfx2[6-1] + V_ts_sq * xfx2[6-3] + V_tb_sq * xfx2[6-5] )
	+ xfx2[6+2] * ( V_ud_sq * xfx1[6+1] + V_us_sq * xfx1[6+3] + V_ub_sq * xfx1[6+5] )
	+ xfx2[6+4] * ( V_cd_sq * xfx1[6+1] + V_cs_sq * xfx1[6+3] + V_cb_sq * xfx1[6+5] )
	+ xfx2[6+6] * ( V_td_sq * xfx1[6+1] + V_ts_sq * xfx1[6+3] + V_tb_sq * xfx1[6+5] );
    }
    return res/x1/x2;
  }
  //
  double lumi_DY(double x1, double x2, double mu, double M, process p, collider c) {
    double res;
    if(p == DYneutral) res = lumi_DYn(x1,x2,mu,M,c);
    else if(p == DYpositive) res = lumi_DYp(x1,x2,mu,M,c);
    else if(p == DYnegative) res = lumi_DYm(x1,x2,mu,M,c);
    else abort();
    return res;
  }




};


